#include "dictsettings.h"
#include "dictwindow.h"
#include "dict.h"

#include <KConfig>


DictConf::DictConf()
{
	config = KGlobal::config();
}

DictConf::~DictConf()
{
	config->sync();
}

void DictConf::read()
{
	KConfigGroup mainConf(config, Dikt::AppName);

	corrections = mainConf.readEntry("Corrections", true);
	headings = mainConf.readEntry("Headings", true);
	matchPage = mainConf.readEntry("MatchPage", true);
	defaultSearch = mainConf.readEntry("DefaultSearch", 0);

	KConfigGroup yankConf(config, "Selections");

	yank = yankConf.readEntry("Yank", false);
	localYank = yankConf.readEntry("Local", true);
	remoteYank = yankConf.readEntry("Remote", true);
	desktopYank = yankConf.readEntry("Desktop", false);

	yankMin = yankConf.readEntry("MinLen", 2);
	yankMax = yankConf.readEntry("MaxLen", 120);
	yankWarn = yankConf.readEntry("WarnLen", 40);
	modifierKey = yankConf.readEntry("ModifierKey", 0);

	reformat = mainConf.readEntry("Reformat", true);
}

void DictConf::write()
{
	KConfigGroup mainConf(config, Dikt::AppName);

	mainConf.writeEntry("Corrections", corrections);
	mainConf.writeEntry("Headings", headings);
	mainConf.writeEntry("MatchPage", matchPage);
	mainConf.writeEntry("DefaultSearch", defaultSearch);

	KConfigGroup yankConf(config, "Selections");

	yankConf.writeEntry("Yank" , yank);
	yankConf.writeEntry("Local" , localYank);
	yankConf.writeEntry("Remote" , remoteYank);
	yankConf.writeEntry("MinLen", yankMin);
	yankConf.writeEntry("MaxLen", yankMax);
	yankConf.writeEntry("WarnLen", yankWarn);
	yankConf.writeEntry("ModifierKey", modifierKey);
	yankConf.writeEntry("Desktop", desktopYank);

	mainConf.writeEntry("Reformat", reformat);
}

void DictConf::writeServer(const QStringList &hosts)
{
	KConfigGroup serverConf(config, "Server");
	serverConf.writeEntry("Hosts", hosts);
	serverConf.writeEntry("Selected", hosts.indexOf(dicthost));
}

void DictConf::writeHost(const QStringList &database, const QString &strategy)
{
	KConfigGroup hostConf(config, dicthost);
	hostConf.writeEntry("Database", database);
	hostConf.writeEntry("Strategy", strategy);
}

QStringList DictConf::server()
{
	KConfigGroup serverConf(config, "Server");
	return serverConf.readEntry("Hosts", QStringList());
}

QString DictConf::host()
{
	KConfigGroup serverConf(config, "Server");
	int selected = serverConf.readEntry("Selected", 0);

	QStringList hosts = server();
	if (selected < 0 || selected >= hosts.size())
		return Dict::DefaultHost;

	return hosts[selected];
}

int DictConf::port()
{
	KConfigGroup hostConf(config, dicthost);
	return hostConf.readEntry("Port", Dict::DefaultPort);
}

QStringList DictConf::database()
{
	KConfigGroup hostConf(config, dicthost);
	return hostConf.readEntry("Database", QStringList());
}

QString DictConf::strategy()
{
	KConfigGroup hostConf(config, dicthost);
	return hostConf.readEntry("Strategy", QString());
}

QString DictConf::username()
{
	KConfigGroup hostConf(config, dicthost);
	return hostConf.readEntry("Username", "");
}

QString DictConf::password()
{
	KConfigGroup hostConf(config, dicthost);
	return hostConf.readEntry("Password", "");
}

void DictConf::writeUser(const QString &username, const QString &password)
{
	KConfigGroup hostConf(config, dicthost);
	hostConf.writeEntry("Username", username);
	hostConf.writeEntry("Password", password);
}

void DictConf::deleteUser()
{
	KConfigGroup hostConf(config, dicthost);
	hostConf.deleteEntry("Username");
	hostConf.deleteEntry("Password");
}

int DictConf::sidebar()
{
	KConfigGroup mainConf(config, Dikt::AppName);
	sidebarSide = mainConf.readEntry("Sidebar", 0);
	if (sidebarSide) sidebarSide = 1;
	return sidebarSide;
}

QList<int> DictConf::widths()
{
	KConfigGroup mainConf(config, Dikt::AppName);
	QList<int> widths;
	widths << 100 << 200;
	return mainConf.readEntry("Widths", widths);
}

bool DictConf::sidebarVisible()
{
	KConfigGroup mainConf(config, Dikt::AppName);
	return mainConf.readEntry("Splitter", false);
}

int DictConf::stack()
{
	KConfigGroup mainConf(config, Dikt::AppName);
	return mainConf.readEntry("Stack", 0);
}

void DictConf::writeSidebar(int index)
{
	KConfigGroup mainConf(config, Dikt::AppName);
	mainConf.writeEntry("Sidebar", sidebarSide);
	mainConf.writeEntry("Stack", index);
}

void DictConf::writeWidths(QList<int> sizes)
{
	KConfigGroup mainConf(config, Dikt::AppName);
	bool visible = sizes[0] && sizes[1];
	if (visible)
		mainConf.writeEntry("Widths", sizes);
	mainConf.writeEntry("Splitter", visible);
}

QString DictConf::font()
{
	KConfigGroup mainConf(config, Dikt::AppName);
	return mainConf.readEntry("Font", "");
}

void DictConf::writeFont(const QString &font)
{
	KConfigGroup mainConf(config, Dikt::AppName);
	mainConf.writeEntry("Font", font);
}


DictSettings::DictSettings(QWidget *parent)
	: KDialog(parent)
{
	setupUi(this);
	setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
	setMainWidget(tab);
	listbox->setFocus();
}

QStringList DictSettings::hosts()
{
	return listbox->items();
}

void DictSettings::setHosts(const QStringList &hosts)
{
	listbox->setItems(hosts);
}

void DictSettings::setFont(const QFont &font)
{
	fontrequester->setFont(font);
}

void DictSettings::readConf(const DictConf &conf)
{
	corrections->setChecked(conf.corrections);
	headings->setChecked(conf.headings);
	matchPage->setChecked(conf.matchPage);
// 	autoMatch->setChecked(conf.autoMatch);
	search->setCurrentIndex(conf.defaultSearch);
	sidebar->setCurrentIndex(conf.sidebarSide);

	yank->setChecked(conf.yank);
	localYank->setChecked(conf.localYank);
	remoteYank->setChecked(conf.remoteYank);
// 	desktopYank->setChecked(conf.desktopYank);

	modifierKey->setCurrentIndex(conf.modifierKey);

	yankMin->setValue(conf.yankMin);
	yankMax->setValue(conf.yankMax);
	yankWarn->setValue(conf.yankWarn);

	reformat->setChecked(conf.reformat);
}

void DictSettings::writeConf(DictConf &conf)
{
	conf.corrections = corrections->isChecked();
	conf.headings = headings->isChecked();
	conf.matchPage = matchPage->isChecked();
// 	conf.autoMatch = autoMatch->isChecked();
	conf.defaultSearch = search->currentIndex();
	conf.sidebarSide = sidebar->currentIndex();

	conf.yank = yank->isChecked();
	conf.localYank = localYank->isChecked();
	conf.remoteYank = remoteYank->isChecked();
// 	conf.desktopYank = desktopYank->isChecked();

	conf.modifierKey = modifierKey->currentIndex();

	conf.yankMin = yankMin->value();
	conf.yankMax = yankMax->value();
	conf.yankWarn = yankWarn->value();

	conf.reformat = reformat->isChecked();
}
