#ifndef DICTWINDOW_H
#define DICTWINDOW_H

#include "dict.h"
#include "dictsettings.h"
#include "dictwidgets.h"

#include <KXmlGuiWindow>

class QSplitter;
class QStackedWidget;
class QComboBox;
class QLabel;
class KSelectAction;

class DictBrowser;
class DictPage;
class DictTracker;
class DictSelection;

namespace Dikt
{
extern const char *AppName;
extern const char *Version;
}

class DictWindow : public KXmlGuiWindow
{
	Q_OBJECT

public:
	DictWindow(const QString &location = "");
	~DictWindow();

public Q_SLOTS:
	void define(const QString &query);
	void match(const QString &query);

protected Q_SLOTS:
	void define();
	void match();
	void define(const QUrl &link);
	void setHost(const QString &host);
	void setDatabaseMenu(const DictList &list);
	void setStrategyMenu(const DictList &list);
	void setQuery(const QString &text) { query = text; };

	void configure();
	void authenticate();
	void intro() { about("about:"); };
	void about(const QString &query);
	void info(const QString &text) { info("about:" + text, text); };
	void info(const QString &query, const QString &text);
	void error(const QString &text, int code);
	void warning(const QString &text, int code);
	void cancel();
	void help() { about("about:help"); };

	void startup(const QString &location);
	void newWindow();
	void save();
	void print();
	void preview();

private:
	static const int maxQueryLen = 120;
	QComboBox *entry;
	DictBrowser *browser, *matchList;
	DictHistory *backwardMenu, *forwardMenu;
	DictInfo *dictinfo;
	KSelectAction *hostMenu;
	DictSelect *strategyMenu;
	DictMenu *databaseMenu;
	DictView *databaseList;
	Dict *dict;

	DictSettings *settings;
	QString query;
	DictSearch *search;
	QSplitter *splitter;
	QStackedWidget *sidebar;
	KAction *stop, *yank, *auth;
	DictStatus *hostStatus, *userStatus;
	DictConf conf;
	DictPage *page;
	DictTracker *tracker;
	DictSelection *selection;

	void setupActions();
	void setupBrowser();
	void setupDict();
	void setupTracker();
	void setupStatusbar();
	void dicturl(const QUrl &url);

	QString dicthost() { return hostMenu->currentText(); };
	QStringList database() { return databaseMenu->current(); };
	QString strategy() { return strategyMenu->current(); };

private Q_SLOTS:
	void readConfig();
	void writeConfig();
	void hostConfig(const QString &host);
	void serverConfig();
	void loadSettings();
	void loadConfig();
	void setHostStatus(bool connected);
	void setUserStatus(bool authenticated);
	void setCapabilities(bool identified);

	void statusMessage(const QString &message);
	void toggleMenubar();
	void setSidebar(int side);
	void setSidebar(QWidget *widget);
	void toggleDatabaseList();
	void toggleMatchList();
	void openMatchList();
	void enableMatchList(bool enable);
	void nextFrame();
	void yankClipboard(const QString &selection);
};

#endif
